import json
import importlib.resources

def create_english_list(path):
    with open(path, 'r') as f:
        lines = f.readlines()
    
    r = set()
    for line in lines:
        line = line.strip()
        if line.isalnum():
            r.add(line)

    return r

class Trie(dict):
    def insert(self, word):
        children = self
        for c in word:
            if c not in children:
                children[c] = Trie()
            children = children[c]
        children['*'] = None

def create_trie(wordlist):
    trie = Trie()
    for word in wordlist:
        trie.insert(word)

    with open('.data/english.json', 'w') as f:
        json.dump(trie, f)

def restore_trie():
    with importlib.resources.open_text('wordhunt_package.data', 'english.json') as f:
        trie = json.load(f)
    return trie


from .english import create_english_list, restore_trie

class Solver:


    def __init__(self, matrix_string, length=4, width=4):
        if len(matrix_string) != length * width or not matrix_string.isalnum():
            raise ValueError("Invalid input string")
        self.trie = restore_trie()
        self.matrix = [[None for j in range(width)] for i in range(length)]
        index = 0
        for i in range(length):
            for j in range(width):
                self.matrix[i][j] = matrix_string[index].lower()
                index += 1
        self.output = set()

    def backtracking(self, i, j, word, path, curr_len, children):
        rows = len(self.matrix)
        cols = len(self.matrix[0])

        if '*' in children:
            self.output.add((word, curr_len))

        for dx, dy in [(1, 0), (-1, 0), (0, 1), (0, -1), (1, 1), (-1, 1), (1, -1), (-1, -1)]:
            xx = i + dx
            yy = j + dy

            if xx < 0 or xx >= rows or yy < 0 or yy >= cols or (xx, yy) in path:
                continue

            element = self.matrix[xx][yy]
            if element not in children:
                continue

            self.backtracking(xx, yy, word + self.matrix[xx][yy], path.union([(xx, yy)]), curr_len + 1, children[element])

        return

    def solve(self):
        rows = len(self.matrix)
        cols = len(self.matrix[0])

        for i in range(rows):
            for j in range(cols):
                self.backtracking(i, j, self.matrix[i][j], set([(i, j)]), 1, self.trie[self.matrix[i][j]])

        return [element[0] for element in sorted(list(self.output), key=lambda x: x[1], reverse=True) if element[1] >= 3]

def run(): 
    while True:
        print("Enter wordhunt board:")
        matrix_string = input("> ")
        if matrix_string in ['quit', 'q', 'stop', 'break']:
            break
        try:
            solver = Solver(matrix_string)
            print(solver.solve())
            break
        except ValueError as e:
            print(e)


from setuptools import setup, find_packages

setup(
    name="wordhunt_package",
    version="0.2.0",
    author="Jeffrey Kim",
    author_email="jkjeffrey7@gmail.com",
    description="Type \"wordhunt\" and enter your Wordhunt board to get all possible words in Wordhunt!",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/jeffreykim/wordhunt-bot",
    packages=find_packages(),
    package_data={
        'wordhunt_package': ['data/english.json'],  # Include all .json files in the 'data' folder
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "wordhunt=wordhunt_package.solver:run",
        ],
    },
)